#!/usr/bin/env node
import { p as parser } from './liquidsoap-a9bd57d1.js';
import parseArgs from 'minimist';
import BluebirdPromise from 'bluebird';
import { glob } from 'glob';
import fs from 'node:fs';
import process from 'node:process';
import '@lezer/lr';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

var Color;
(function (Color) {
    Color[Color["Red"] = 31] = "Red";
    Color[Color["Green"] = 32] = "Green";
    Color[Color["Yellow"] = 33] = "Yellow";
})(Color || (Color = {}));
function colorize(value, color) {
    return "\u001b[" + color + "m" + String(value) + "\u001b[39m";
}
function focusedNode(cursor) {
    const { type, from, to } = cursor;
    return { type, from, to };
}
function printTree(tree, input, options = {}) {
    let errors = [];
    const cursor = tree.cursor();
    const { from = 0, to = input.length, start = 0, includeParents = false, } = options;
    let output = "";
    const prefixes = [];
    for (;;) {
        const node = focusedNode(cursor);
        let leave = false;
        if (node.from <= to && node.to >= from) {
            const enter = !node.type.isAnonymous &&
                (includeParents || (node.from >= from && node.to <= to));
            if (enter) {
                leave = true;
                const isTop = output === "";
                if (!isTop || node.from > 0) {
                    output += (!isTop ? "\n" : "") + prefixes.join("");
                    const hasNextSibling = cursor.nextSibling() && cursor.prevSibling();
                    if (hasNextSibling) {
                        output += " ├─ ";
                        prefixes.push(" │  ");
                    }
                    else {
                        output += " └─ ";
                        prefixes.push("    ");
                    }
                }
                if (node.type.isError)
                    errors = [...errors, node];
                output += node.type.isError
                    ? colorize(node.type.name, Color.Red)
                    : node.type.name;
            }
            const isLeaf = !cursor.firstChild();
            if (enter) {
                const hasRange = node.from !== node.to;
                output +=
                    " " +
                        (hasRange
                            ? "[" +
                                colorize(start + node.from, Color.Yellow) +
                                ".." +
                                colorize(start + node.to, Color.Yellow) +
                                "]"
                            : colorize(start + node.from, Color.Yellow));
                if (hasRange && isLeaf) {
                    output +=
                        ": " +
                            colorize(JSON.stringify(input.slice(node.from, node.to)), Color.Green);
                }
            }
            if (!isLeaf)
                continue;
        }
        for (;;) {
            if (leave)
                prefixes.pop();
            leave = cursor.type.isAnonymous;
            if (cursor.nextSibling())
                break;
            if (!cursor.parent())
                return { errors, output };
            leave = true;
        }
    }
}
const run = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _, q } = parseArgs(process.argv.slice(2), { boolean: ["q"] });
        if (!_.length) {
            console.error("No filename passed!");
            process.exit(1);
        }
        let exitCode = 0;
        const files = yield glob(_);
        yield BluebirdPromise.each(files, (file) => __awaiter(void 0, void 0, void 0, function* () {
            const input = "" + fs.readFileSync(file);
            const tree = parser.parse(input);
            console.log(`Parsing ${file}..`);
            const { errors, output } = printTree(tree, input);
            if (!q)
                console.log(output);
            if (errors.length) {
                exitCode = 1;
                console.error(`Encountered ${errors.length} error(s) while parsing ${file}!`);
            }
            errors.forEach((node) => {
                const start = input.lastIndexOf("\n", node.from) || 0;
                const stop = input.indexOf("\n", node.to) || input.length;
                const lineStart = input.slice(0, start).split("\n").length + 1;
                if (node.to === node.from) {
                    console.log(`Line ${lineStart}, char ${node.from - start}: Error!`);
                    return;
                }
                console.log(`Line ${lineStart}, error: ` +
                    input.slice(start, node.from) +
                    colorize(input.slice(node.from, node.to), Color.Red) +
                    input.slice(node.to, stop));
            });
        }));
        process.exit(exitCode);
    }
    catch (err) {
        console.error(`Error while running liquidsoap-lezer-print-tree: ${err}`);
        process.exit(1);
    }
});
run();

export { printTree };
